/*
 * matgen.c
 *	Generate random matrices. 
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <time.h>

#include <util.h>

/*
 * matinit()
 *	a[i,j] = seed+i+j;
 */
void
matinit(int *A, uint N, int seed)
{
	uint i, j;
	
	for (i=0; i<N; ++i) {
		for (j=0; j<N; ++j) {
			*A++ = seed+i+j;
		}
	}
}

/*
 * matwrite()
 * 	file <- A
 */
int
matwrite(char *path, void *A, int N)
{
	int fd, r;
	uint sz = N*N*sizeof(int);

	fd = open(path, O_WRONLY|O_CREAT, S_IREAD|S_IWRITE);
	if (fd == -1) {
		perror("matwrite, open");
		exit(1);
	}
	r = write(fd, A, sz); 
	if (r == -1) {
		perror("matwrite, write");
		exit(1);
	} else if (r != sz) {
		fprintf(stderr, 
		        "Warning: Only %u of %u bytes have been written!\n",
		        r, sz);
	}
	close(fd);
	return(0);
}

/*
 * main()
 */
int
main(int argc, char **argv)
{
	ulong matsz, npgs; 
	char *matname;
	void *mem;
	int seed = 0;

	/*
	 * Process arguments
	 */	
	if (argc < 3) {
		printf("Usage: %s N file_name [seed]\n", argv[0]);
		printf("\tN: number of rows and columns for square matrix\n");
		exit(1);
	}
	argv++; argc--; 
	matsz = atoi(*argv);
	if (matsz == 0) {
		printf("N must be > 0\n");
		exit(1);
	}
	argv++; argc--; 
	matname = *argv;
	argv++; argc--; 
	if (argc > 0) {
		seed = atoi(*argv);
		argv++; argc--; 
	}

	/* region must hold 3 matsz x matsz matrices*/
	npgs = btop(sizeof(int) * matsz * matsz + (NBPG - 1));

	printf("matsz %lu, npgs %lu (0x%lx)\n", matsz, npgs, npgs);

	/*
	 * Get a new virtual memory region of appropiate size
	 */
	printf("Allocating memory...\n");
	mem = (void *)mmap(0, ptob(npgs), PROT_READ|PROT_WRITE,
		MAP_ANON|MAP_PRIVATE, 0, 0);
	if (mem == MAP_FAILED) {
		perror("matgen, mmap");
		return(1);
	}
	printf("...memory successfully allocated at %p\n", mem);

	/*
	 * Create MAT 
	 */
	printf("Initializing MAT...\n");
	matinit(mem, matsz, seed);
	printf("Writing MAT...\n");
	matwrite(matname, mem, matsz);

	printf("Bye!\n");
	
	return(0);
}


